module.exports = bootstrap;

var async = require('async');
var nano = require('nano');

var assureConnection = require('./lib/assure-connection');
var fixAdminParty = require('./lib/fix-admin-party');
var runBootstrap = require('./lib/bootstrap');

/**
 * Bootstapping mNutrition CouchDB
 *
 * 1. sets config
 * 2. creates databases (unless existent)
 * 3. sets database `_security` settings
 * 4. creates test user accounts (unless already existent)
 * 5. creates design documents
 **/
function bootstrap (options, callback) {
  var state = {
    config: options.couchdb,
    bootstrapPath: options.bootstrapPath,
    couch: getCouchConnection(options.couchdb),
    adminCouch: getAdminCouchConnection(options.couchdb),
    log: options.log || console.log.bind(console)
  };

  async.waterfall([
    assureConnection.bind(null, state),
    fixAdminParty.bind(null, state),
    runBootstrap.bind(null, state)
  ], callback);
}

function getCouchConnection (config) {
  return nano({
    url: config.url,
    parseUrl: false
  });
}

function getAdminCouchConnection (config) {
  var url = config.url;
  var user = config.adminUser;
  var pass = config.adminPass;

  // add admin username & password to url
  user = encodeURIComponent(user);
  pass = encodeURIComponent(pass);
  url = url.replace(/:\/\//, '://' + user + ':' + pass + '@');

  return nano({
    url: url,
    parseUrl: false
  });
}
