module.exports = bootstrap;

var kazanaConfig = require('kazana-config');
var couchdbBootstrap = require('couchdb-bootstrap');

function bootstrap (state, next) {
  couchdbBootstrap(state.adminCouch, state.bootstrapPath, {
    mapDbName: {
      'raw-data': kazanaConfig.dbRawData,
      'integrated-data': kazanaConfig.dbIntegratedData
    }
  }, function (error, response) {
    if (error) {
      return next(error);
    }

    if ('configure' in response && response.configure && response.configure.length) {
      state.log(['bootstrap'], 'config updated');
    }

    if ('secure' in response) {
      Object.keys(response.secure).forEach(function (dbname) {
        state.log(['bootstrap'], '_security set on database ' + dbname + '.');
      });
    }

    if ('push' in response) {
      Object.keys(response.push).forEach(function (dbname) {
        if (dbname === '_users') {
          var usernames = response.push._users.map(function (resp) {
            return resp.id.split(':')[1];
          });

          return state.log(['bootstrap'], 'users created or updated: ' + usernames.join(', '));
        }

        var counts = response.push[dbname].reduce(function (memo, resp) {
          if (resp.id.match(/^_design/)) {
            memo.ddocs++;
          } else {
            memo.docs++;
          }
          return memo;
        }, {
          ddocs: 0,
          docs: 0
        });

        if (counts.ddocs) {
          state.log(['bootstrap'], '_design docs added or updated on database ' + dbname + ': ' + counts.ddocs + '.');
        }
        if (counts.docs) {
          state.log(['bootstrap'], 'docs added or updated on database ' + dbname + ': ' + counts.docs + '.');
        }
      });
    }

    next(null);
  });
}
